@@ -1,27 +1,27 @@
 #!/usr/bin/python3
 # This Python file uses the following encoding: utf-8
 from PyQt5 import QtCore, QtWidgets, uic, QtGui
-from PyQt5.QtWidgets import QInputDialog, QFileDialog, QApplication, QMainWindow, QSpinBox, QCheckBox, QMessageBox, QPushButton
+from PyQt5.QtWidgets import QFileDialog, QMainWindow, QMessageBox, QPushButton
 from functools import partial
 
 import signal
 import sys
-import qencoder
+
 from qencoder.mainwindow import Ui_qencoder
 from qencoder.pav1n import Av1an
 
 import traceback
 
-import multiprocessing
-from threading import Thread
-
-from types import SimpleNamespace
 from pathlib import Path
 import os
+from time import sleep
+import psutil
+from multiprocessing import Process, Queue
 
 import pickle
 
-#baseUIClass, baseUIWidget = uic.loadUiType("mainwindow.ui")
+# baseUIClass, baseUIWidget = uic.loadUiType("mainwindow.ui")
+
 
 class window(QMainWindow, Ui_qencoder):
     twopassState = True
@@ -32,6 +32,8 @@ class window(QMainWindow, Ui_qencoder):
     encodeList = []
     currentFile = ""
     scenedetectFailState = -1
+    currentlyRunning = 0
+    killFlag = 0
     if 'APPDATA' in os.environ:
         confighome = os.environ['APPDATA']
     elif 'XDG_CONFIG_HOME' in os.environ:
@@ -45,6 +47,7 @@ class window(QMainWindow, Ui_qencoder):
         self.setupUi(self)
         self.inputFileChoose.clicked.connect(self.inputFileSelect)
         self.outputFileChoose.clicked.connect(self.outputFileSelect)
+        self.pushButton_vmafmodel.clicked.connect(self.inputVmafSelect)
         self.label_audio.setEnabled(0)
         self.spinBox_quality.setValue(26)
         enable_slot = partial(self.audioEnableState, self.checkBox_audio)
@@ -81,6 +84,8 @@ class window(QMainWindow, Ui_qencoder):
         self.actionSave_Queue.triggered.connect(self.saveQueueAuto)
         self.actionSave_Queue_As.triggered.connect(self.saveQueueTo)
         self.actionOpen_Queue.triggered.connect(self.openQueueFrom)
+        self.actionSave_Preset.triggered.connect(self.savePresetAs)
+        self.actionOpen_Preset.triggered.connect(self.openPresetFrom)
         self.pushButton_save.setEnabled(0)
         self.pushButton_save.clicked.connect(self.saveToQueue)
         self.tabWidget.currentChanged[int].connect(self.setCustomText)
@@ -88,8 +93,10 @@ class window(QMainWindow, Ui_qencoder):
         self.pushButton_down.clicked.connect(self.queueMoveDown)
         self.pushButton_del.clicked.connect(self.removeFromQueue)
         self.pushButton_edit.clicked.connect(self.editCurrentQueue)
-        self.actionSave_Preset.triggered.connect(self.savePresetAs)
-        self.actionOpen_Preset.triggered.connect(self.openPresetFrom)
+        self.checkBox_cropping.clicked.connect(self.enableCropping)
+        self.checkBox_rescale.clicked.connect(self.enableRescale)
+        self.checkBox_vmaf.clicked.connect(self.enableDisableVmaf)
+        self.checkBox_lessshitsplit.clicked.connect(self.enableDisableGoodSplit)
         if (len(sys.argv) > 1):
             self.inputPath.setText(sys.argv[1])
 
@@ -121,20 +128,57 @@ class window(QMainWindow, Ui_qencoder):
             0: ["", "--color-space=unknown"],
             1: ["--color-primaries=bt709 --transfer-characteristics=bt709 --matrix-coefficients=bt709", "--color-space=bt709"],
             2: ["--color-primaries=bt601 --transfer-characteristics=bt601 --matrix-coefficients=bt601", "--color-space=bt601"],
-            3: ["--color-primaries=bt2020 --transfer-characteristics=bt2020-10bit --matrix-coefficients=bt2020ncl", "--color-space=bt2020"],
-            4: ["--color-primaries=bt2020 --transfer-characteristics=bt2020-10bit --matrix-coefficients=bt2020cl", "--color-space=bt2020"]
+            3: ["--color-primaries=bt2020 --transfer-characteristics=smpte2084 --matrix-coefficients=bt2020ncl", "--color-space=bt2020"],
         }
 
         try:
             filehandler = open(self.configpath, 'rb')
             settings = pickle.load(filehandler)
             self.setFromPresetDict(settings)
+            self.enableCropping()
+            self.enableRescale()
+            self.enableDisableVmaf()
+            self.enableDisableGoodSplit()
         except:
             print("Unable to load existing preset at: " + str(self.configpath) + ".")
             print("Possibly the first time you have run this, corrupted, or an older version")
             print("Do not report this")
         # self.speedButton.changeEvent.connect(self.setSpeed)
 
+    def enableDisableGoodSplit(self):
+        state = self.checkBox_lessshitsplit.isChecked()
+        self.label_split.setEnabled(not state)
+        self.spinBox_split.setEnabled(not state)
+
+    def enableDisableVmaf(self):
+        state = self.checkBox_vmaf.isChecked()
+        self.label_vmafpath.setEnabled(state)
+        self.pushButton_vmafmodel.setEnabled(state)
+        self.label_qmin.setEnabled(state)
+        self.label_target.setEnabled(state)
+        self.label_teststeps.setEnabled(state)
+        self.spinBox_vmafsteps.setEnabled(state)
+        self.spinBox_minq.setEnabled(state)
+        self.doubleSpinBox_vmaf.setEnabled(state)
+        self.spinBox_boost.setEnabled(not state)
+        self.label_boost.setEnabled(not state)
+        self.spinBox_maxq.setEnabled(state)
+        self.label_maxq.setEnabled(state)
+
+    def enableRescale(self):
+        state = self.checkBox_rescale.isChecked()
+        self.spinBox_xres.setEnabled(state)
+        self.spinBox_yres.setEnabled(state)
+        self.label_xres.setEnabled(state)
+        self.label_yres.setEnabled(state)
+
+    def enableCropping(self):
+        state = self.checkBox_cropping.isChecked()
+        self.spinBox_croptop.setEnabled(state)
+        self.spinBox_cropdown.setEnabled(state)
+        self.spinBox_cropleft.setEnabled(state)
+        self.spinBox_cropright.setEnabled(state)
+
     def editCurrentQueue(self):
         if (self.listWidget.currentRow() <= -1):
             return
@@ -150,6 +194,10 @@ class window(QMainWindow, Ui_qencoder):
             self.pushButton_save.setEnabled(1)
             del self.encodeList[self.listWidget.currentRow()]
             self.redrawQueueList()
+            self.enableCropping()
+            self.enableRescale()
+            self.enableDisableVmaf()
+            self.enableDisableGoodSplit()
 
     def disableMinimumSplitBox(self, state):
         if state:
@@ -160,6 +208,7 @@ class window(QMainWindow, Ui_qencoder):
             self.spinBox_minsplit.setEnabled(1)
 
     def encodeFinished(self, success):
+        self.currentlyRunning = 0
         if success:
             self.pushButton.setEnabled(1)
             self.pushButton.setText("Finalize")
@@ -207,7 +256,8 @@ class window(QMainWindow, Ui_qencoder):
         file_pi = open(self.configpath, 'wb')
         pickle.dump(curSettings, file_pi)
         file_pi.close()
-        os.killpg(0, signal.SIGTERM)
+        if not sys.platform.startswith('win'):
+            os.killpg(0, signal.SIGTERM)
         event.accept()
 
     def saveQueueAuto(self):
@@ -242,6 +292,7 @@ class window(QMainWindow, Ui_qencoder):
         self.encodeList = pickle.load(filehandler)
         self.currentFile = filename[0]
         self.redrawQueueList()
+        self.tabWidget.setCurrentIndex(5)
 
     def queueMoveUp(self):
         if (self.listWidget.currentRow() > 0):
@@ -334,9 +385,9 @@ class window(QMainWindow, Ui_qencoder):
         #Get indexes of current colorspace comboBox
         inputSpace = self.comboBox_colorspace.currentIndex()
         # if colorspace index is 5 uses a custom value set by the user
-        if (inputSpace == 5):
+        if (inputSpace == 4):
             return self.lineEdit_colordata.text()
-        #return empty string if current encoder combobox indexes is 2 
+        #return empty string if current encoder combobox indexes is 2
         if (self.comboBox_encoder.currentIndex() == 2):
             return ""
         #else map the colorspace index into a list of av1,vp9 color space then return the appropriate string based on the encoder
@@ -461,8 +512,12 @@ class window(QMainWindow, Ui_qencoder):
         else:
             return self.spinBox_quality.value()
 
+    def inputVmafSelect(self):
+        filename = QFileDialog.getOpenFileName(filter = "VMAF models(*.pkl *.model);;All(*)")
+        self.label_vmafpath.setText(filename[0])
+
     def inputFileSelect(self):
-        filename = QFileDialog.getOpenFileName(filter = "Videos(*.mp4 *.mkv *.webm *.flv *.gif *.3gp *.wmv *.avi);;All(*)")
+        filename = QFileDialog.getOpenFileName(filter = "Videos(*.mp4 *.mkv *.webm *.flv *.gif *.3gp *.wmv *.avi *.y4m);;All(*)")
         self.inputPath.setText(filename[0])
         if (len(self.outputPath.text()) > 1):
             self.pushButton.setEnabled(1)
@@ -494,6 +549,8 @@ class window(QMainWindow, Ui_qencoder):
 
     def bitrateEnableState(self, checkbox):
         self.label_q.setText("Bitrate (kbps)")
+        self.checkBox_vmaf.setChecked(0)
+        self.checkBox_vmaf.setEnabled(0)
         self.spinBox_quality.setMaximum(99999)
         self.spinBox_quality.setMinimum(8)
         self.spinBox_quality.setValue(3000)
@@ -505,6 +562,7 @@ class window(QMainWindow, Ui_qencoder):
 
     def bitrateDisableState(self, checkbox):
         self.label_q.setText("CRF")
+        self.checkBox_vmaf.setEnabled(1)
         self.spinBox_quality.setMaximum(63)
         self.spinBox_quality.setMinimum(0)
         self.spinBox_quality.setValue(30)
@@ -516,7 +574,22 @@ class window(QMainWindow, Ui_qencoder):
     def getFFMPEGParams(self):
         if (self.checkBox_ffmpegcmd.isChecked()):
             return self.textEdit_ffmpegcmd.toPlainText()
-        return ""
+        astr = ""
+        addedStuff = False
+        if (self.checkBox_cropping.isChecked() and (self.spinBox_cropdown.value() > 0 or self.spinBox_cropright.value() > 0 or self.spinBox_croptop.value() > 0 or self.spinBox_cropleft.value() > 0)):
+            widthSub = self.spinBox_cropright.value() + self.spinBox_cropleft.value()
+            heightSub = self.spinBox_croptop.value() + self.spinBox_cropdown.value()
+            astr += "-filter:v \"crop=iw-" + str(widthSub) + ":ih-" + str(heightSub) + ":" + str(self.spinBox_cropleft.value()) + ":" + str(self.spinBox_croptop.value())
+            addedStuff = True
+        if (self.checkBox_rescale.isChecked()):
+            if (addedStuff):
+                astr += ",scale=" + str(self.spinBox_xres.value()) + ":" + str(self.spinBox_yres.value())
+            else:
+                astr += "-filter:v \"scale=" + str(self.spinBox_xres.value()) + ":" + str(self.spinBox_yres.value())
+                addedStuff = True
+        if (addedStuff) :
+            astr += "\""
+        return astr
 
     def getVideoParams(self):
         if (self.checkBox_videocmd.isChecked()):
@@ -608,6 +681,20 @@ class window(QMainWindow, Ui_qencoder):
         self.spinBox_minsplit.setValue(dict['minsplitsize'])
         self.spinBox_maxkfdist.setValue(dict['maxkfdist'])
 
+        # 1.5 variables
+        self.comboBox_inputFormat.setCurrentIndex(dict['inputFmt'])
+        self.comboBox_colorspace.setCurrentIndex(dict['colordataCS'])
+        self.lineEdit_colordata.setText(dict['colordataText'])
+        self.checkBox_vmaf.setChecked(dict['isTargetVMAF'])
+        self.spinBox_minq.setValue(dict['TargetVMAFMinQ'])
+        self.spinBox_maxq.setValue(dict['TargetVMAFMaxQ'])
+        self.spinBox_vmafsteps.setValue(dict['TargetVMAFSteps'])
+        self.doubleSpinBox_vmaf.setValue(dict['TargetVMAFValue'])
+        self.label_vmafpath.setText(dict['TargetVMAFPath'])
+        self.checkBox_shutdown.setChecked(dict['ShutdownAfter'])
+        self.checkBox_lessshitsplit.setChecked(dict['BetterSplittingAlgo'])
+        self.checkBox_unsafeSplit.setChecked(dict['unsafe_split'])
+
     def getPresetDict(self):
         return {'2p': self.checkBox_twopass.isChecked(), 'audio': self.checkBox_audio.isChecked(), 'enc': self.comboBox_encoder.currentIndex(),
                 'aq' : self.audioqualitybox.currentIndex(), 'preset': self.presetbox.currentIndex(),
@@ -621,7 +708,14 @@ class window(QMainWindow, Ui_qencoder):
                 'cusvid': self.checkBox_videocmd.isChecked(), 'cusaud': self.checkBox_audiocmd.isChecked(),
                 'cusffmpeg': self.checkBox_ffmpegcmd.isChecked(), 'vidcmd': self.textEdit_videocmd.toPlainText(),
                 'audcmd': self.textEdit_audiocmd.toPlainText(), 'ffmpegcmd': self.textEdit_ffmpegcmd.toPlainText(),
-                'minsplitsize': self.spinBox_minsplit.value(), 'maxkfdist': self.spinBox_maxkfdist.value()
+                'minsplitsize': self.spinBox_minsplit.value(), 'maxkfdist': self.spinBox_maxkfdist.value(),
+                'inputFmt': self.comboBox_inputFormat.currentIndex(), 'colordataCS': self.comboBox_colorspace.currentIndex(),
+                'colordataText': self.lineEdit_colordata.text(), 'isTargetVMAF' : self.checkBox_vmaf.isChecked(),
+                'TargetVMAFMinQ': self.spinBox_minq.value(), 'TargetVMAFMaxQ': self.spinBox_maxq.value(),
+                'TargetVMAFSteps': self.spinBox_vmafsteps.value(), 'TargetVMAFValue': self.doubleSpinBox_vmaf.value(),
+                'TargetVMAFPath': self.label_vmafpath.text(), 'ShutdownAfter': self.checkBox_shutdown.isChecked(),
+                'BetterSplittingAlgo': self.checkBox_lessshitsplit.isChecked(),
+                'unsafe_split': self.checkBox_unsafeSplit.isChecked()
         }
 
 
@@ -629,15 +723,28 @@ class window(QMainWindow, Ui_qencoder):
         args = {'video_params': self.getVideoParams(), 'input_file': Path(self.inputPath.text()), 'encoder': 'aom',
                 'workers' : self.spinBox_jobs.value(), 'audio_params': self.getAudioParams(),
                 'threshold': self.spinBox_split.value(),
-                'temp': Path(os.path.abspath("temp_" + Path(self.outputPath.text()).parts[-1])),
                 'logging' : None, 'passes' : (2 if self.checkBox_twopass.isChecked() else 1),
                 'output_file': Path(self.outputPath.text()), 'scenes' : None,
                 'resume' : self.checkBox_resume.isChecked(), 'keep' : self.checkBox_tempfolder.isChecked(),
                 'min_splits' : self.checkBox_minsplit.isChecked(), 'pix_format' : self.comboBox_inputFormat.currentText(),
-                'ffmpeg_cmd' : self.getFFMPEGParams(), 'min_split_dist' : self.spinBox_minsplit.value()
+                'ffmpeg_cmd' : self.getFFMPEGParams(), 'min_split_dist' : self.spinBox_minsplit.value(),
+                'use_vmaf' : self.checkBox_vmaf.isChecked(), 'threads' : self.spinBox_threads.value(),
+                'better_split' : self.checkBox_lessshitsplit.isChecked(), 'cpuused' : self.spinBox_speed.value(),
+                'unsafe_split' : self.checkBox_unsafeSplit.isChecked()
         }
+        args['temp'] = Path(str(os.path.splitext(self.outputPath.text())[0]) + "/temp_" + str(os.path.splitext(self.outputPath.text())[1]))
 
-        if (self.checkBox_bitrate.isChecked() or self.spinBox_boost.value() < 1):
+        if (self.checkBox_vmaf.isChecked()):
+            args['vmaf_steps'] = self.spinBox_vmafsteps.value()
+            args['min_cq'] = self.spinBox_minq.value()
+            args['max_cq'] = self.spinBox_maxq.value()
+            args['vmaf_target'] = self.doubleSpinBox_vmaf.value()
+            args['vmaf_path'] = self.label_vmafpath.text()
+
+        if (self.checkBox_minsplit.isChecked()):
+            args['min_split_dist'] = 0
+
+        if (self.checkBox_bitrate.isChecked() or self.spinBox_boost.value() < 1 or self.checkBox_vmaf.isChecked()):
             args['boost'] = False
             args['br'] = 0
             args['bl'] = 0
@@ -647,6 +754,7 @@ class window(QMainWindow, Ui_qencoder):
             args['br'] = self.spinBox_boost.value()
         if (self.comboBox_encoder.currentIndex() >= 1):
             args['encoder'] = 'vpx'
+            args['better_split'] = False
         return args
 
     def encodeVideoQueue(self):
@@ -655,12 +763,11 @@ class window(QMainWindow, Ui_qencoder):
             return
         self.encodeVideo1()
         self.runningEncode = True
-        self.worker = EncodeWorker(self.encodeList)
+        self.worker = EncodeWorker(self.encodeList, self, self.checkBox_shutdown.isChecked())
         self.workerThread = QtCore.QThread()
         self.worker.updateQueuedStatus.connect(self.updateQueuedStatus)
         self.worker.updateStatusProgress.connect(self.updateStatusProgress)
         self.worker.encodeFinished.connect(self.encodeFinished)
-        self.worker.sceneDetectFailed.connect(self.sceneDetectFailed)
         self.worker.moveToThread(self.workerThread)  # Move the Worker object to the Thread object
         self.workerThread.started.connect(self.worker.run)  # Init worker run() at startup (optional)
         self.workerThread.start()
@@ -673,20 +780,37 @@ class window(QMainWindow, Ui_qencoder):
         self.encodeVideo1()
         print("Running in non-queued mode with a single video")
         self.runningEncode = True
-        self.worker = EncodeWorker([args], self)
+        self.worker = EncodeWorker([args], self, self.checkBox_shutdown.isChecked())
         self.workerThread = QtCore.QThread()
         self.worker.updateQueuedStatus.connect(self.updateQueuedStatus)
         self.worker.updateStatusProgress.connect(self.updateStatusProgress)
         self.worker.encodeFinished.connect(self.encodeFinished)
-        self.worker.sceneDetectFailed.connect(self.sceneDetectFailed)
         self.worker.moveToThread(self.workerThread)  # Move the Worker object to the Thread object
         self.workerThread.started.connect(self.worker.run)  # Init worker run() at startup (optional)
         self.workerThread.start()
 
     def encodeVideo1(self):
         if (self.runningEncode):
+            if (self.currentlyRunning):
+                buttonReply = QMessageBox.question(self, 'Stop encode?',
+                                                   "Warning: You may lose encoding progress.", QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
+                if buttonReply != QMessageBox.Yes:
+                    return
+
             self.finalizeEncode()
             return
+        print("Writing current settings to config")
+        curSettings = self.getPresetDict()
+        file_pi = open(self.configpath, 'wb')
+        pickle.dump(curSettings, file_pi)
+        file_pi.close()
+        self.actionOpen.setEnabled(0)
+        self.actionSave.setEnabled(0)
+        self.actionSave_Queue.setEnabled(0)
+        self.actionSave_Queue_As.setEnabled(0)
+        self.actionOpen_Queue.setEnabled(0)
+        self.actionSave_Preset.setEnabled(0)
+        self.actionOpen_Preset.setEnabled(0)
         self.label_3.setEnabled(0)
         self.label_threads.setEnabled(0)
         self.spinBox_threads.setEnabled(0)
@@ -696,8 +820,8 @@ class window(QMainWindow, Ui_qencoder):
         self.textEdit_videocmd.setEnabled(0)
         self.textEdit_audiocmd.setEnabled(0)
         self.textEdit_ffmpegcmd.setEnabled(0)
-
-        self.pushButton.setEnabled(0)
+        self.currentlyRunning = True
+        self.pushButton.setText("Cancel")
         self.pushButton_encQueue.setEnabled(0)
         self.pushButton_save.setEnabled(0)
         self.progressBar_total.setEnabled(1)
@@ -749,9 +873,41 @@ class window(QMainWindow, Ui_qencoder):
         self.label_maxkfdist.setEnabled(0)
         self.spinBox_maxkfdist.setEnabled(0)
         self.spinBox_minsplit.setEnabled(0)
+        self.pushButton_edit.setEnabled(0)
+        self.checkBox_cropping.setEnabled(0)
+        self.checkBox_rescale.setEnabled(0)
+        self.label_xres.setEnabled(0)
+        self.label_yres.setEnabled(0)
+        self.spinBox_yres.setEnabled(0)
+        self.spinBox_xres.setEnabled(0)
+        self.spinBox_cropleft.setEnabled(0)
+        self.spinBox_cropright.setEnabled(0)
+        self.spinBox_croptop.setEnabled(0)
+        self.spinBox_cropdown.setEnabled(0)
+        self.checkBox_vmaf.setEnabled(0)
+        self.label_boost.setEnabled(0)
+        self.label_qmin.setEnabled(0)
+        self.label_teststeps.setEnabled(0)
+        self.label_target.setEnabled(0)
+        self.pushButton_vmafmodel.setEnabled(0)
+        self.label_vmafpath.setEnabled(0)
+        self.spinBox_minq.setEnabled(0)
+        self.spinBox_vmafsteps.setEnabled(0)
+        self.doubleSpinBox_vmaf.setEnabled(0)
+        self.spinBox_maxq.setEnabled(0)
+        self.label_maxq.setEnabled(0)
+        self.checkBox_shutdown.setEnabled(0)
+        self.checkBox_lessshitsplit.setEnabled(0)
+        self.checkBox_unsafeSplit.setEnabled(0)
 
     def finalizeEncode(self):
+        self.workerThread.quit()
+        self.workerThread.wait(2000)
+        self.killFlag = True
+        self.workerThread.wait()
+        self.killFlag = False
         self.runningEncode = False
+        self.currentlyRunning = False
         self.pushButton.setStyleSheet("")
         self.pushButton.setText("▶  Encode")
         self.label_threads.setEnabled(1)
@@ -759,7 +915,6 @@ class window(QMainWindow, Ui_qencoder):
         self.checkBox_audio.setEnabled(1)
         self.spinBox_speed.setEnabled(1)
         self.spinBox_speed.setValue(self.spinBox_speed.value())
-        self.spinBox_split.setEnabled(1)
         self.spinBox_jobs.setEnabled(1)
         self.label_jobs.setEnabled(1)
         self.inputFileChoose.setEnabled(1)
@@ -770,7 +925,12 @@ class window(QMainWindow, Ui_qencoder):
         self.spinBox_threads.setEnabled(1)
         self.label_2.setEnabled(1)
         self.label_q.setEnabled(1)
-        self.label_split.setEnabled(1)
+        self.checkBox_lessshitsplit.setEnabled(1)
+        self.checkBox_shutdown.setEnabled(1)
+        if (not self.checkBox_lessshitsplit.isChecked()):
+            self.label_split.setEnabled(1)
+            self.spinBox_split.setEnabled(1)
+
         self.label_inputformat.setEnabled(1)
         self.label_6.setEnabled(1)
         self.label_5.setEnabled(1)
@@ -792,8 +952,11 @@ class window(QMainWindow, Ui_qencoder):
         self.spinBox_quality.setEnabled(1)
         self.checkBox_bitrate.setEnabled(1)
         self.label_preset.setEnabled(1)
+        self.enableDisableVmaf()
         if (not self.checkBox_bitrate.isChecked()):
+            self.checkBox_vmaf.setEnabled(1)
             self.label_boost.setEnabled(1)
+            self.spinBox_boost.setEnabled(1)
             self.comboBox_quality.setEnabled(1)
             self.label_quality.setEnabled(1)
         if (not self.checkBox_rtenc.isChecked()):
@@ -816,6 +979,7 @@ class window(QMainWindow, Ui_qencoder):
         self.pushButton_up.setEnabled(1)
         self.pushButton_down.setEnabled(1)
         self.pushButton_del.setEnabled(1)
+        self.pushButton_edit.setEnabled(1)
         self.listWidget.setEnabled(1)
         self.pushButton_save.setEnabled(0)
         self.progressBar_total.setValue(0)
@@ -824,51 +988,37 @@ class window(QMainWindow, Ui_qencoder):
         self.label_maxkfdist.setEnabled(1)
         self.spinBox_maxkfdist.setEnabled(1)
         if (not self.checkBox_minsplit.isChecked()):
-            self.spinBox_minsplit.setEnabled(0)
-            self.label_minsplit.setEnabled(0)
-
+            self.spinBox_minsplit.setEnabled(1)
+            self.label_minsplit.setEnabled(1)
+        self.actionOpen.setEnabled(1)
+        self.actionSave.setEnabled(1)
+        self.actionSave_Queue.setEnabled(1)
+        self.actionSave_Queue_As.setEnabled(1)
+        self.actionOpen_Queue.setEnabled(1)
+        self.actionSave_Preset.setEnabled(1)
+        self.actionOpen_Preset.setEnabled(1)
+        self.checkBox_cropping.setEnabled(1)
+        self.checkBox_rescale.setEnabled(1)
+        self.checkBox_unsafeSplit.setEnabled(1)
+        self.enableCropping()
+        self.enableRescale()
+        self.enableDisableVmaf()
+        self.enableDisableGoodSplit()
         print("Enabled all buttons, returning program to normal")
 
-    def sceneDetectFailed(self):
-        msgbox = QMessageBox()
-        msgbox.setIcon(QMessageBox.Warning)
-        msgbox.setText("Scenedetect splitting failed. This could be because your input video format is not supported, your operating system (eg if you are on Windows it can sometimes fail), your ffmpeg version may be wrong, or your input video may be corrupted. Reencoding first may fix some of these issues, alternatively we could avoid splitting, or do minimal splitting based only on video length.")
-        msgbox.setWindowTitle("Scenedetect failed!")
-        reencodeButton = QPushButton()
-        reencodeButton.setText("Reencode")
-        msgbox.addButton(reencodeButton, QMessageBox.ActionRole)
-        nosplitButton = QPushButton()
-        nosplitButton.setText("Do not split")
-        msgbox.addButton(nosplitButton, QMessageBox.ActionRole)
-        minsplitButton = QPushButton()
-        minsplitButton.setText("Minimal time based splitting")
-        msgbox.addButton(minsplitButton, QMessageBox.ActionRole)
-        msgbox.addButton(QMessageBox.Abort)
-        msgbox.exec()
-
-        if (msgbox.clickedButton() == reencodeButton):
-            print("Reencoding")
-            self.scenedetectFailState = 1
-        elif (msgbox.clickedButton() == nosplitButton):
-            print("no splitting")
-            self.scenedetectFailState = 3
-        elif (msgbox.clickedButton() == minsplitButton):
-            print("minsplitting")
-            self.scenedetectFailState = 2
-        else:
-            print("Aborting")
-            self.scenedetectFailState = 0
-
 class EncodeWorker(QtCore.QObject):
     updateStatusProgress = QtCore.pyqtSignal(str, int)
     updateQueuedStatus = QtCore.pyqtSignal(str)
     encodeFinished = QtCore.pyqtSignal(bool)
-    sceneDetectFailed = QtCore.pyqtSignal()
+    runningPav1n = False
 
-    def __init__(self, argdata, window):
+    def __init__(self, argdata, window, shutdown):
         super().__init__()
         self.argdat = argdata
         self.window = window
+        self.shutdown = shutdown
+        self.q = Queue()
+        self.istty = sys.stdin.isatty()
 
     def runProcessing(self, dictargs):
         self.window.scenedetectFailState = -1
@@ -877,24 +1027,63 @@ class EncodeWorker(QtCore.QObject):
         av1an.main_thread(self)
         print("\n\nEncode completed for " + str(dictargs['input_file']) + " -> " + str(dictargs['output_file']))
 
-    def run(self):
+    def internalRunner(self):
+        print("Running")
         if (len(self.argdat) > 1):
-            self.updateQueuedStatus.emit("Encoding video 1/" + str(len(self.argdat)))
+            self.q.put([1, "Encoding video 1/" + str(len(self.argdat))])
         try:
             for i in range(len(self.argdat)):
                 if (len(self.argdat) > 1):
-                    self.updateStatusProgress.emit("Processing video " + str(i + 1), 0)
+                    self.q.put([0, "Processing video " + str(i + 1), 0])
+                    self.runningPav1n = True
                     self.runProcessing(self.argdat[i][0])
-                    self.updateQueuedStatus.emit("Completed encode " + str(i + 1) + "/" + str(len(self.argdat)))
+                    while (self.runningPav1n):
+                        sleep(0.1)
+                    self.q.put([1, "Completed encode " + str(i + 1) + "/" + str(len(self.argdat))])
                 else:
-                    self.updateStatusProgress.emit("Processing video", 0)
+                    self.q.put([0, "Processing video", 0])
+                    self.runningPav1n = True
                     self.runProcessing(self.argdat[i][0])
+                    while (self.runningPav1n):
+                        sleep(0.1)
 
             if (len(self.argdat) > 1):
-                self.updateQueuedStatus.emit("Completed video queue")
-            self.encodeFinished.emit(True)
+                self.q.put([1, "Completed video queue"])
+            self.q.put([2, 1])
         except Exception as e:
             print(e)
             traceback.print_exc()
-            self.encodeFinished.emit(False)
+            self.q.put([2, 0])
+        if (self.shutdown):
+            if (sys.platform.startswith('win')):
+                os.system('shutdown -s')
+            else:
+                try:
+                    os.system('systemctl poweroff')
+                except Exception as e:
+                    # If that doesn't work we can try shutting down the "other" way
+                    os.system("shutdown now -h")
 
+    def run(self):
+        t = Process(target = self.internalRunner)
+        t.start()
+        while (t.is_alive()):
+            sleep(0.05)
+            if(self.window.killFlag):
+                print("Killing all children processes. Hopefully this works.")
+                parent = psutil.Process(t.pid)
+                for child in parent.children(recursive=True):  # or parent.children() for recursive=False
+                    child.kill()
+                parent.kill()
+                t.join()
+            try:
+                qdat = self.q.get(False)
+                if (qdat[0] == 0):
+                    self.updateStatusProgress.emit(qdat[1], qdat[2])
+                if (qdat[0] == 1):
+                    self.updateQueuedStatus.emit(qdat[1])
+                if (qdat[0] == 2):
+                    self.encodeFinished.emit(qdat[1])
+            except Exception as e:
+                pass
+        self.q.put([0, "Encode completed!", 100])
